<html>
  <head>
    <title>Stuff</title>
    <style>
     #hoverable:hover {
       font-style: italic;
     }
    </style>
  </head>
  <body>
    <p>
      Hello {% if name %}to {{ name }}{% else %}world{% endif %}, from &#39;me&#39; &amp; &quot;friends&quot;!
    </p>

    <p id="hoverable">Hover me!</p>
  </body>
</html>
